`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// LIP_5x7matrix: Pontmtrix kijelz hasznlata kzvetlen bitkp zemmdban
// Az adatbemenet egy 35 x 1 bites vltoz a teljes kijelzmez szmra, 
// 5 db 7 bitesv adategysgben megadva. Ez a 7 bites adategysg fekv 
// (landscape, 7x5 pont) kijelznl egy teljes sort, ll kijelznl 
// (portr, 5x7 pont) egy teljes oszlopot jelent, az els esetben a bal fels 
// sarokbl kezdve, sorfolytonosan balrl jobbra, a msodikban a jobb als 
// sarokbl, alulrl felfel haladva, osloponknt nvekv indexelssel, 
// a kvetkez minta szerint
// |0 1 2 3 4 5 6|7 8 9 10 11 12 13|14 15 16 17 18 19 20|...|28 29 30 31 32 33 34|
// |1. oszl./sor |  2. oszlop/sor  |  3. oszlop/sor     |4. |    5. oszlop/sor   |
// Ennek megfeleleen a kvetkez kijelzkp rtelemzsek lehetsgesek:
// ( A folytonos vonalak a krtya krvonalai, az orientci jelzshez)
//        Fekv elrendezs:                               ll elrendezs:
//     __________________________________              |
//     |                                               |    6 13 20 27 34
//     |   0  1  2  3  4  5  6                         |    5 12 19 26 33
//     |   7  8  9 10 11 12 13                         |    4 11 18 25 32
//     |  14 15 16 17 18 19 20                         |    3 10 17 24 31
//     |  21 22 23 24 25 26 27                         |    2  9 16 23 30
//     |  28 29 30 31 32 33 34                         |    1  8 15 22 29
//     |                                               |    0  7 14 21 28 
//                                                     |__________________________   
// A fekv elrendezsben a (0,0) index pixel a bal fels sarokelem, mg az ll
// elrendezsben a bal als elem. Fekv elrendezsnl a kijelzs soronknt trtnik,
// 1. sor, 2. sor, 3. sor, 4. sor, 5. sor, ll elrendezsben 1. oszlop, 2. oszlop,
// 3. oszlop, 4. oszlop, 5. oszlop. A 35 bites teljes bitkpbl az rtelemzs 
// szerinti 7 bites sor/oszlop adategysgek megjelentse idmultiplex mdon 
// trtnik, krlbell 1kHz krli kapcsolsi frekvencival
//////////////////////////////////////////////////////////////////////////////////
module LIP_5x7matrix(
            input  clk16M,          // 16MHz frekvencij rajel
            input  rst,             // Szinkron RESET jel
            input  [ 0:34] bitmap,  // 35 bites adat bemenet (teljes bitkp)
            output [ 7: 0] pixels,  // Kimeneti 8 bites adategysg, az MSb 0
            output [ 4: 0] sel      // Multiplexelt kivlaszt kimenetek
                    );


///////////////////////////////////////////////////////////////////////////////////
// A lptets frekvencija megkzelten 1kHz, amit a 16MHz-es rajelbl lltunk el
// A binris oszt minden 16384-ik rajelben elllt egy lptet jelet
///////////////////////////////////////////////////////////////////////////////////

reg [13:0] cnt;							// rajeloszt szmll
wire tc  = &cnt;							// Vgrtk pulzus kiadsa, ha minden bit magas


always @ (posedge clk16M)
   if (rst)  cnt <= 14'b0;          // Alapllapot belltsa
	else      cnt <= cnt + 1;        // Szmlls felfel
 
//////////////////////////////////////////////////////////////////////////////////
// Az idmultiplex vezrlst egy tbites shiftregiszter (gyrs szmll) vezrli, 
// a bitkp megjelentse 7 bites egysgenknt (fekv kpnl soronknt, 
// fellrl lefel, ll kpnl oszloponknt, balrl jobbra) trtnik. 
// Ha select=10000, akkor a select[0] vonal aktv s a bitmap[ 0: 6] biteken lv rtk
//  jelenik meg a legfels sorban / bal oldali oszlopban.
// Ha select=01000, akkor a select[1] vonal aktv s a bitmap[ 7:13] biteken lv rtk 
//  jelenik meg a kvetkez soron/oszlopon
//  s gy tovbb...
///////////////////////////////////////////////////////////////////////////////////

reg [4:0] select=5'b10000;

always @ (posedge clk16M)
   if (rst)       select <= 5'b10000;                  // Kezdllapot, select[4] aktv
	else if (tc)   select <= {select[0], select[4:1]};  // Adott temben lp a kivlaszts


//////////////////////////////////////////////////////////////////////////////////                                                            
// A select jelekkel vlaszthatunk a bemeneti 35 bites adat bitmezi kzl, 
// az adott megjelentsi fzisnak megfelelen
// Elosztott multiplexer hlzat, a hatkony erforrs kihasznlshoz
// Egy 8 bites 5->1-be adatkivlasztst valst meg, AND-OR hlzattal
//////////////////////////////////////////////////////////////////////////////////

assign pixels[7:0] = ({8{select[4]}} & {1'b0,bitmap[ 0: 6]}) | 
                     ({8{select[3]}} & {1'b0,bitmap[ 7:13]}) | 
                     ({8{select[2]}} & {1'b0,bitmap[14:20]}) | 
                     ({8{select[1]}} & {1'b0,bitmap[21:27]}) |
                     ({8{select[0]}} & {1'b0,bitmap[28:34]}); 

assign sel = select;

endmodule 


